@@ -34,6 +34,8 @@ void activate_action_next (GtkAction *action, gpointer data);
 void activate_action_prev (GtkAction *action, gpointer data);
 void activate_action_eject (GtkAction *action, gpointer data);
 void activate_action_reload (GtkAction *action, gpointer data);
+void activate_action_manual (GtkAction *action, gpointer data);
+void activate_action_shortcuts (GtkAction *action, gpointer data);
 void activate_action_about (GtkAction *action, gpointer data);
 void activate_action_quit (GtkAction *action, gpointer data);
 void activate_action_view_toolbar (GtkAction *action, gpointer data);
