@@ -1,9 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 /*
- *  File-Roller
+ *  Goo
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,5 +26,12 @@
 #include <gtk/gtkwindow.h>
 
 extern GtkWindow *main_window;
+extern int AutoPlay;
+extern int HideShow;
+
+void system_notify (const char *title,
+		    const char *msg,
+		    int         x,
+		    int         y);
 
 #endif /* MAIN_H */
